# Copyright (C) 2016 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from . import (
    all_others,
    analysis_options,
    birth_and_death,
    dynamic_options,
    fe_body_loads,
    fe_constraints,
    fe_forces,
    fe_surface_loads,
    gap_conditions,
    inertia,
    load_step_operations,
    load_step_options,
    master_dof,
    miscellaneous_loads,
    multi_field_solver_convergence_controls,
    multi_field_solver_definition_commands,
    multi_field_solver_global_controls,
    multi_field_solver_interface_mapping,
    multi_field_solver_load_transfer,
    multi_field_solver_time_controls,
    nonlinear_options,
    ocean,
    radiosity,
    rezoning,
    solid_body_loads,
    solid_constraints,
    solid_forces,
    solid_surface_loads,
    solution_status,
    spectrum_options,
    twod_to_3d_analysis,
)
